/*
 * You own your strategy, and you have the right to modify, share or keep your strategy secret.
 * The license below is meant to protect Inovance from any liability and provide you the most 
 * freedom with your strategy.
 *
 * Copyright (c) 2015, Inovance, Inc.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of TRAIDE nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
//+------------------------------------------------------------------+
//|                                                   Inovance, Inc. |
//|                                     https://www.inovancetech.com |
//+------------------------------------------------------------------+

#property copyright ""
#property link      "https://www.inovancetech.com"
#property description "TRAIDE Exported Strategy"
#property version   "1.00"
#property strict

enum ExitCondition {
    BAR_CLOSE,    // Bar Close
    TP_SL         // TP/SL
};

// EA Inputs
input ExitCondition exitCondition = TP_SL;    // Exit Conditions
extern double   stopLoss           = 20.0;        // Stop Loss (pips)
extern double   takeProfit         = 20.0;        // Take profit (pips)
extern bool     useTrailingStop    = 0;           // Trailing Stop?
input  double   lots               = 0.01;         // Lots
extern int      slippage           = 5;           // Maximum Slippage (pips)
input  int     magicNumber        = 1456501042;  // Magic Number


// Other global variables
double buyStopLossPips    = 0.0;
double sellStopLossPips   = 0.0;
double buyTakeProfitPips  = 0.0;
double sellTakeProfitPips = 0.0;
datetime lastBar          = D'1970.01.01 00:00';

int OnInit() {
    if (Bars < 30) {
        Print("Error initializing: Not enough bars in chart");
        return INIT_FAILED;
    }    
    if (exitCondition == TP_SL) {
        if (takeProfit < 10) {
            Print("Error initializing: Take Profit less than 10");
            return INIT_PARAMETERS_INCORRECT;
        }
        if(stopLoss < 10) {
            Print("Error initializing: Stop Loss less than 10");
            return INIT_PARAMETERS_INCORRECT;
        }
        if (Digits == 5 || Digits == 3) {
            takeProfit *= 10;
            stopLoss *= 10;
        }
    }    
    if (Digits == 5 || Digits == 3) {          
        slippage *= 10;
    }           
    Print("Exit Condition=", exitCondition, " Lots=", lots, " Take Profit=", takeProfit, " Stop Loss=", stopLoss, " Slippage=", slippage); 
    return INIT_SUCCEEDED;
}

void OnTick() {
    if (exitCondition == TP_SL && useTrailingStop && isPositionOpen()) {
        modifyOrderTrailingStop();
    }

    if (isNewBar()) {
        if (exitCondition == TP_SL) {
            setTakeProfitAndStopLoss();
        }
        int signal = traideRule0() + traideRule1();
        processSignal(signal);
    }
}

bool isNewBar() {
    if (lastBar != Time[0]) {
        lastBar = Time[0];
        return TRUE;
    }
    return FALSE;
} 

void modifyOrderTrailingStop() {
    double trailingStop;
    bool result;
    for (int i = 0; i < OrdersTotal(); i++) {
        if (isPositionOpen()) {
            switch (OrderType()) {
            case OP_BUY:
                trailingStop = NormalizeDouble(Ask - stopLoss * Point, Digits);
                if (trailingStop > OrderStopLoss()) {
                    result = OrderModify(OrderTicket(), OrderOpenPrice(), trailingStop, OrderTakeProfit(), 0, CLR_NONE);
                    if (!result) {
                        Print("Error calling OrderModify. Error code=",GetLastError());
                    }
                }
                break;
            case OP_SELL:     
                trailingStop = NormalizeDouble(Bid + stopLoss * Point, Digits);
                if (trailingStop < OrderStopLoss()) {
                    result = OrderModify(OrderTicket(), OrderOpenPrice(), trailingStop, OrderTakeProfit(), 0, CLR_NONE);
                    if (!result) {
                        Print("Error calling OrderModify. Error code=",GetLastError());
                    }
                }
                break;
            default:
                Print("Error modifyOrderTrailingStop, unexpected OrderType: ", OrderType());
                break;
            }
        }
    }
}

void setTakeProfitAndStopLoss() {
    buyStopLossPips = NormalizeDouble(Ask - stopLoss * Point, Digits);
    sellStopLossPips = NormalizeDouble(Bid + stopLoss * Point, Digits);
    buyTakeProfitPips = NormalizeDouble(Ask + takeProfit * Point, Digits);
    sellTakeProfitPips = NormalizeDouble(Bid - takeProfit * Point, Digits); 
}

void processSignal(const int signal) {
    if (isPositionOpen()) {
       processSignalHasPositionOpen(signal);
    } else {
       processSignalNoPositionOpen(signal);
    }
}

bool isPositionOpen() {
    for (int i = 0; i < OrdersTotal(); i++) {
        if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderMagicNumber() == magicNumber && OrderSymbol() == Symbol()) { 
            return true;
       }
    }
    return false;
}


void processSignalHasPositionOpen(const int signal) {
    switch (OrderType()) {
    case OP_BUY:   // A buy order exists
        if (signal == 0) {
            Print("Long Conditions No Longer Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Bid, slippage, Violet)) {
                Print("Error closing buy order: ", GetLastError());
            }
        } else if (signal < 0) {
            Print("Short Conditions Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Bid, slippage, Violet)) {
            Print("Error closing buy order: ", GetLastError());
            } else if (OrderSend(Symbol(), OP_SELL, lots, Bid, slippage, sellStopLossPips, sellTakeProfitPips, "TRAIDE EA", magicNumber, 0, Red) < 0) {
                Print("Error opening sell order: ", GetLastError());
            }
        } else if (signal > 0) {
            Print("Long Conditions Still Met");
        }
        break;
    case OP_SELL:  // A sell order exists
        if (signal == 0) {
            Print("Short Conditions No Longer Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Ask, slippage, Violet)) {
                Print("Error closing sell order: ", GetLastError());
            }
        } else if (signal > 0) {
            Print("Long Conditions Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Ask, slippage, Violet)) {
                Print("Error closing sell order: ", GetLastError());
            } else if (OrderSend(Symbol(), OP_BUY, lots, Ask, slippage, buyStopLossPips, buyTakeProfitPips, "TRAIDE EA", magicNumber, 0, Green) < 0) {
                Print("Error opening buy order: ", GetLastError());
            }
        } else if (signal < 0) {
            Print("Short Conditions Still Met");
        }
        break;
    default:
         Print("Error processSignalHasPositionOpen, unexpected OrderType: ", OrderType());
         break;
    }
}

void processSignalNoPositionOpen(const int signal) {
    if (signal > 0) {
        Print("Long Conditions Met");
        if (OrderSend(Symbol(), OP_BUY, lots, Ask, slippage, buyStopLossPips, buyTakeProfitPips, "TRAIDE EA", magicNumber, 0, Green) < 0) {
            Print("Error opening buy order: ", GetLastError());
        }
    } else if (signal < 0) {
        Print("Short Conditions Met");
        if (OrderSend(Symbol(), OP_SELL, lots, Bid, slippage, sellStopLossPips, sellTakeProfitPips, "TRAIDE EA", magicNumber, 0, Red) < 0) {
            Print("Error opening sell order: ", GetLastError());
        }
    }
}

void OnDeinit(const int reason) {
    switch (reason) {
    case REASON_PROGRAM:
        Print("Closing EA: Expert Advisor terminated its operation by calling the ExpertRemove() function (REASON_PROGRAM).");
        closeEA();
        break;
    case REASON_REMOVE:
        Print("Closing EA: Program has been deleted from the chart (REASON_REMOVE).");
        closeEA();
        break;
    case REASON_RECOMPILE:
        Print("Reinitializing EA: Program has been recompiled (REASON_RECOMPILE).");
        reinitEA();
        break;
    case REASON_CHARTCHANGE:
        Print("Reinitializing EA: Symbol or chart period has been changed (REASON_CHARTCHANGE).");
        reinitEA();
        break;
    case REASON_CHARTCLOSE:
        Print("Closing: Chart has been closed (REASON_CHARTCLOSE).");
        closeEA();
        break;
    case REASON_PARAMETERS:
        Print("Reinitializing EA: Input parameters have been changed by user (REASON_PARAMETERS).");
        reinitEA();
        break;
    case REASON_ACCOUNT:
        Print("Reinitializing EA: Another account has been activated or reconnection to the trade server has occurred due to changes in the account settings (REASON_ACCOUNT).");
        reinitEA();
        break;
    case REASON_TEMPLATE:
         Print("Reinitializing EA: A new template has been applied (REASON_TEMPLATE).");
         reinitEA();
         break;
    case REASON_INITFAILED:
         Print("Closing EA: OnInit() handler has returned a nonzero value.");
         closeEA();
         break;
    case REASON_CLOSE:
         Print("Closing EA: Terminal has been closed.");
         closeEA();
         break;
    default:
         Print("Error, unexpected OnDeInit reason: ", reason);
         break;
    }
}

void closeEA() {
    if (isPositionOpen()) {
        switch (OrderType()) {
        case OP_BUY:   // A buy order exists
            if (!OrderClose(OrderTicket(), OrderLots(), Bid, slippage, Violet)) {
                Print("Error closing buy order: ", GetLastError());
            }
            break;
        case OP_SELL:  // A sell order exists
            if (!OrderClose(OrderTicket(), OrderLots(), Ask, slippage, Violet)) {
                Print("Error closing sell order: ", GetLastError());
            }
            break;
        default:
             Print("Error closeEA, unexpected OrderType: ", OrderType());
             break;
        }
    }    
}

void reinitEA() {
    reinitGlobals();
    if (OnInit() != INIT_SUCCEEDED) {
        Print("Closing EA: Reinitializing failed.");
        closeEA();
    }
}

void reinitGlobals() {
    buyStopLossPips = 0.0;
    buyTakeProfitPips = 0.0;
    sellStopLossPips = 0.0;
    sellTakeProfitPips = 0.0;
    lastBar = D'1970.01.01 00:00';
}


// Produced by TRAIDE Strategy Code Generator
int indCCI0period = 200;
int traideRule0() {
    double CCI0 = NormalizeDouble(iCustom(NULL, PERIOD_M1, "TRAIDE-Indicators\\Technical\\Momentum\\CCI", indCCI0period, 0, 1), 5);
    int match = 0;
    if (CCI0 > 0) match++;
    if (match == 1) return 1;
    return 0;
}
int traideRule1() {
    double CCI0 = NormalizeDouble(iCustom(NULL, PERIOD_M1, "TRAIDE-Indicators\\Technical\\Momentum\\CCI", indCCI0period, 0, 1), 5);
    int match = 0;
    if (CCI0 < 0) match++;
    if (match == 1) return -1;
    return 0;
}
